"use strict";

const mapObj = require("map-obj");

let GrammarError = require("./grammar-error");
let compiler = require("./compiler");
let parser = require("./parser");

module.exports = {
	// PEG.js version (uses semantic versioning).
	VERSION: require("../package.json").version,

	GrammarError: GrammarError,
	parser: parser,
	compiler: compiler,

	// Generates a parser from a specified grammar and returns it.
	//
	// The grammar must be a string in the format described by the metagramar in
	// the parser.pegjs file.
	//
	// Throws |peg.parser.SyntaxError| if the grammar contains a syntax error or
	// |peg.GrammarError| if it contains a semantic error. Note that not all
	// errors are detected during the generation and some may protrude to the
	// generated parser and cause its malfunction.
	generate(grammar, options = {}) {
		// FIXME: Validatem
		
		let config = {
			parser: parser,
			passes: mapObj(compiler.passes, (stage, passesForStage) => {
				return [ stage, Object.values(passesForStage) ];
			})
		};

		return compiler.compile(
			config.parser.parse(grammar),
			config.passes,
			options
		);
	}
};
